{% extends "base.html" %}

{% block content %}
    <ul class="top-level-list">
        {% for env in view_envs %}
        <li>
            <h4 style="color:#666">
                <span>{{'Environment: ' + env['environment']}}</span>
            </h4>
            <ul>
                {% for title in env['checks'] %}
                <li>
                    {{check_result(env['checks'][title], title, env['environment'], is_admin, context)}}
                </li>
                {% endfor %}
            </ul>
        </li>
        {% endfor %}
    </ul>
{% endblock %}
